package com.ether.dao.plot;

import java.util.Date;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import com.ether.PersistenceException;
import com.ether.dao.DomainAccessObjectImpl;
import com.ether.plot.Plot;

public class PlotDAOImpl extends DomainAccessObjectImpl<Plot, Long> implements PlotDAO {
	
	public PlotDAOImpl() {
		super(Plot.class);
	}
	
	@Nullable
	public List<String> getPlotsByDateByInstrumentByStation(@NotNull final String instrument, @NotNull final Date pFormatDate_deb, @NotNull final Date pFormatDate_fin, @NotNull final String station) 
		throws PersistenceException 
	{
		
		final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class,"plot")
			.addOrder(Order.asc("plot.name"))
			.add(Restrictions.ge("datedeb", pFormatDate_deb))
			.add(Restrictions.lt("datedeb", pFormatDate_fin))
            .addOrder( Order.desc( "datedeb" ) )
			.createCriteria("instrument","instrument").add(Restrictions.ilike("instrument.name",instrument,MatchMode.ANYWHERE))
			.createCriteria("station","station").add(Restrictions.ilike("station.name",station,MatchMode.ANYWHERE));
		
		
		
		//select pl_name de nd_plot
		criteria.setProjection(Projections.property("plot.name"));
		
		return selectAllByCriteria(String.class, criteria);
	}

    @Nullable
    public List<String> getPlotsByDateByInstrumentByStationByType(@NotNull final String instrument, @NotNull final Date pFormatDate_deb, @NotNull final Date pFormatDate_fin, @NotNull final String station, @NotNull final String type)
            throws PersistenceException
        {
            String vtype = type;

            final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class,"plot")
                .addOrder(Order.asc("plot.name"))
                .add(Restrictions.ge("datedeb", pFormatDate_deb))
                .add(Restrictions.lt("datedeb", pFormatDate_fin))
                .createCriteria("instrument","instrument").add(Restrictions.ilike("instrument.name",instrument,MatchMode.ANYWHERE))
                .createCriteria("station","station").add(Restrictions.ilike("station.name",station,MatchMode.ANYWHERE))
                .add(Restrictions.ilike("plot.name",vtype,MatchMode.ANYWHERE));


            //select pl_name de nd_plot
            criteria.setProjection(Projections.property("plot.name"));

            return selectAllByCriteria(String.class, criteria);
        }

	@Nullable
	public List<String> getPlotsByDateByInstrument(@NotNull final String instrument, @NotNull final Date pFormatDate_deb, @NotNull final Date pFormatDate_fin) 
		throws PersistenceException 
	{
		final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class);
		criteria.add(Restrictions.ge("datedeb", pFormatDate_deb));
		criteria.add(Restrictions.lt("datedeb", pFormatDate_fin));
		criteria.createCriteria("instrument").add(Restrictions.like("name",instrument));		

		//select pl_name de nd_plot
		criteria.setProjection(Projections.property("name"));
	
		return selectAllByCriteria(String.class, criteria);
	}

	@Nullable
	public List<Date> getDatesByInstrument(@NotNull final String instrument) 
		throws PersistenceException 
	{
		final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class);
		criteria.addOrder(Order.asc("datedeb"));
		criteria.createCriteria("instrument").add(Restrictions.like("name",instrument));
	
		//select pl_name de nd_plot
		criteria.setProjection(Projections.property("datedeb"));
	
		return selectAllByCriteria(Date.class, criteria);
	}

	@Nullable
	public Date getLastDateByInstrumentByStation(@NotNull final String instrument, @NotNull final String station) 
		throws PersistenceException 
	{
		final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class,"plot")
			.createCriteria("instrument","instrument").add(Restrictions.ilike("instrument.name",instrument,MatchMode.ANYWHERE))
			.createCriteria("station", "station").add(Restrictions.ilike("station.name",station,MatchMode.ANYWHERE));
	
		criteria.setProjection(Projections.property("plot.datedeb"));
		criteria.addOrder(Order.asc("plot.datedeb"));
	
		//	return selectByCriteria(Date.class, criteria);
		return (Date)criteria.getExecutableCriteria(getSessionFactory().getCurrentSession()).setMaxResults(1).uniqueResult();
	}


    @Nullable
	public List<Date> getListeDistinctDates() 
		throws PersistenceException 
	{
		//correspond au select from
		final DetachedCriteria criteria = DetachedCriteria.forClass(Plot.class)
			.setProjection(Projections.distinct(Projections.property("datedeb")));
	
		return selectAllByCriteria(Date.class, criteria);
	}
}
